/*******************************************************************************
 * Copyright 2011 John Casey
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package org.commonjava.tensor;

import java.io.PrintStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

public class MultiTensorException
    extends TensorException
{

    private static final long serialVersionUID = 1L;

    private final List<TensorException> errors = new ArrayList<TensorException>();

    public MultiTensorException( final List<TensorException> errors, final String message, final Object... params )
    {
        super( message, params );
        if ( errors != null )
        {
            this.errors.clear();
            this.errors.addAll( errors );
        }
    }

    @Override
    public String getMessage()
    {
        final StringBuilder sb = new StringBuilder( super.getMessage() );
        sb.append( " (" )
          .append( errors.size() )
          .append( " embedded errors):\n" );

        int i = 0;
        for ( final TensorException error : errors )
        {
            sb.append( "\n  " )
              .append( i++ )
              .append( ". " )
              .append( error.getMessage() );
        }

        return sb.toString();
    }

    @Override
    public void printStackTrace()
    {
        printStackTrace( System.err );
    }

    @Override
    public void printStackTrace( final PrintStream err )
    {
        super.printStackTrace( err );

        int i = 0;
        for ( final TensorException error : errors )
        {
            err.printf( "\n\n  %d. ", i++ );
            error.printStackTrace( err );
        }

        err.println();
    }

    @Override
    public void printStackTrace( final PrintWriter err )
    {
        super.printStackTrace( err );

        int i = 0;
        for ( final TensorException error : errors )
        {
            err.printf( "\n\n  %d. ", i++ );
            error.printStackTrace( err );
        }

        err.println();
    }

}
