package org.commonjava.tensor.data;

import java.io.PrintStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.List;

import org.commonjava.tensor.TensorException;

public class MultiTensorDataException
    extends TensorDataException
{

    private static final long serialVersionUID = 1L;

    private final List<TensorDataException> errors = new ArrayList<TensorDataException>();

    public MultiTensorDataException( final List<TensorDataException> errors, final String message,
                                     final Object... params )
    {
        super( message, params );
        if ( errors != null )
        {
            this.errors.clear();
            this.errors.addAll( errors );
        }
    }

    @Override
    public String getMessage()
    {
        final StringBuilder sb = new StringBuilder( super.getMessage() );
        sb.append( " (" )
          .append( errors.size() )
          .append( " embedded errors):\n" );

        int i = 0;
        for ( final TensorException error : errors )
        {
            sb.append( "\n  " )
              .append( i++ )
              .append( ". " )
              .append( error.getMessage() );
        }

        return sb.toString();
    }

    @Override
    public void printStackTrace()
    {
        printStackTrace( System.err );
    }

    @Override
    public void printStackTrace( final PrintStream err )
    {
        super.printStackTrace( err );

        int i = 0;
        for ( final TensorException error : errors )
        {
            err.printf( "\n\n  %d. ", i++ );
            error.printStackTrace( err );
        }

        err.println();
    }

    @Override
    public void printStackTrace( final PrintWriter err )
    {
        super.printStackTrace( err );

        int i = 0;
        for ( final TensorException error : errors )
        {
            err.printf( "\n\n  %d. ", i++ );
            error.printStackTrace( err );
        }

        err.println();
    }

}
