package org.commonjava.tensor.data;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Pattern;

import org.apache.maven.graph.common.ref.ProjectRef;
import org.apache.maven.graph.common.ref.ProjectVersionRef;
import org.apache.maven.graph.effective.EProjectGraph;
import org.apache.maven.graph.effective.EProjectRelationships;
import org.apache.maven.graph.effective.filter.ProjectRelationshipFilter;
import org.apache.maven.graph.effective.ref.EProjectKey;
import org.apache.maven.graph.effective.rel.ProjectRelationship;

public interface TensorDataManager
{

    // FIXME: Rename this metadata key!!
    String MODEL_ERRORS = "aproxTensor-modelErrors";

    String ERROR_SEPARATOR = Pattern.quote( "_::--::_" );

    String PROJECT_RELATIONSHIPS_CACHE = "tensor-effective-project-relationships";

    String GRAPH_CACHE = "tensor-project-graphs";

    Set<ProjectRelationship<?>> storeGraph( EProjectGraph graph )
        throws TensorDataException;

    Set<ProjectRelationship<?>> storeRelationships( EProjectRelationships relationships )
        throws TensorDataException;

    Set<ProjectRelationship<?>> storeRelationships( ProjectRelationship<?>... relationships )
        throws TensorDataException;

    Set<ProjectRelationship<?>> storeRelationships( Collection<ProjectRelationship<?>> relationships )
        throws TensorDataException;

    EProjectGraph getProjectGraph( EProjectKey key )
        throws TensorDataException;

    EProjectGraph getProjectGraph( ProjectVersionRef ref )
        throws TensorDataException;

    List<ProjectVersionRef> getAncestry( ProjectVersionRef source )
        throws TensorDataException;

    ProjectVersionRef getParent( ProjectVersionRef source )
        throws TensorDataException;

    Set<ProjectVersionRef> getKnownChildren( ProjectVersionRef parent )
        throws TensorDataException;

    Set<ProjectRelationship<?>> getAllDirectRelationshipsWithExactSource( ProjectVersionRef source,
                                                                          ProjectRelationshipFilter filter )
        throws TensorDataException;

    Set<ProjectRelationship<?>> getAllDirectRelationshipsWithExactTarget( ProjectVersionRef target,
                                                                          ProjectRelationshipFilter filter )
        throws TensorDataException;

    Set<ProjectRelationship<?>> getAllDirectRelationshipsWithGATarget( ProjectRef target,
                                                                       ProjectRelationshipFilter filter )
        throws TensorDataException;

    //    EProjectRelationships getProjectRelationships( EProjectKey key )
    //        throws TensorDataException;
    //
    //    EProjectRelationships getProjectRelationships( ProjectVersionRef projectRef )
    //        throws TensorDataException;

    //    void reindex()
    //        throws TensorDataException;

    boolean contains( ProjectVersionRef ref );

    Set<ProjectVersionRef> getAllStoredProjectRefs()
        throws TensorDataException;

    Map<String, String> getMetadata( EProjectKey key );

    void addMetadata( EProjectKey key, String name, String value );

    void addMetadata( EProjectKey key, Map<String, String> metadata );

    Set<ProjectVersionRef> getIncompleteSubgraphsFor( ProjectVersionRef ref )
        throws TensorDataException;

    Set<ProjectVersionRef> getVariableSubgraphsFor( ProjectVersionRef ref )
        throws TensorDataException;

    Set<ProjectVersionRef> getIncompleteSubgraphsFor( EProjectKey key )
        throws TensorDataException;

    Set<ProjectVersionRef> getVariableSubgraphsFor( EProjectKey key )
        throws TensorDataException;

    Set<ProjectVersionRef> getAllIncompleteSubgraphs()
        throws TensorDataException;

    Set<ProjectVersionRef> getAllVariableSubgraphs()
        throws TensorDataException;

    void addError( final ProjectVersionRef ref, final Throwable error )
        throws TensorDataException;

    Set<String> getErrors( final ProjectVersionRef ref )
        throws TensorDataException;

    boolean hasErrors( final ProjectVersionRef ref )
        throws TensorDataException;

    Map<ProjectVersionRef, Set<String>> getAllProjectErrors()
        throws TensorDataException;

    Map<ProjectVersionRef, Set<String>> getProjectErrorsInGraph( ProjectVersionRef ref )
        throws TensorDataException;

    void reindex( ProjectVersionRef ref )
        throws TensorDataException;

    void reindexAll()
        throws TensorDataException;

}