package org.commonjava.tensor.io.json;

import java.lang.reflect.Type;
import java.util.Collection;
import java.util.Set;

import org.apache.maven.graph.effective.EProjectCycle;
import org.apache.maven.graph.effective.EProjectGraph;
import org.apache.maven.graph.effective.ref.EProjectKey;
import org.apache.maven.graph.effective.rel.ProjectRelationship;
import org.apache.maven.graph.spi.GraphDriverException;
import org.apache.maven.graph.spi.effective.EGraphDriver;

import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParseException;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;

public class EProjectGraphSer
    extends AbstractProjectNetSer<EProjectGraph>
    implements JsonSerializer<EProjectGraph>, JsonDeserializer<EProjectGraph>
{

    private final EGraphDriver driver;

    public EProjectGraphSer( final EGraphDriver driver )
    {
        this.driver = driver;
    }

    @Override
    public EProjectGraph deserialize( final JsonElement src, final Type typeInfo, final JsonDeserializationContext ctx )
        throws JsonParseException
    {
        final JsonObject obj = src.getAsJsonObject();

        final EProjectKey key = deserializeKey( obj, ctx );
        final Collection<ProjectRelationship<?>> rels = deserializeRelationships( obj, ctx );

        final Set<EProjectCycle> cycles = deserializeCycles( obj, ctx );

        try
        {
            return new EProjectGraph.Builder( key, driver.newInstance() ).withExactRelationships( rels )
                                                                         .withCycles( cycles )
                                                                         .build();
        }
        catch ( final GraphDriverException e )
        {
            throw new JsonParseException( "Cannot get graph database sub-driver: " + e.getMessage(), e );
        }
    }

    @Override
    public JsonElement serialize( final EProjectGraph src, final Type typeInfo, final JsonSerializationContext ctx )
    {
        final JsonObject obj = new JsonObject();
        obj.add( SerializationConstants.EPROJECT_KEY, ctx.serialize( src.getKey() ) );

        serializeRelationships( src, obj, ctx );
        serializeCycles( src, obj, ctx );

        return obj;
    }

}
