package org.commonjava.tensor.io.json;

import java.lang.reflect.Type;

import org.apache.maven.graph.common.ref.ProjectVersionRef;
import org.apache.maven.graph.effective.ref.EGraphFacts;
import org.apache.maven.graph.effective.ref.EProjectKey;

import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParseException;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;

public class EProjectKeySer
    implements JsonSerializer<EProjectKey>, JsonDeserializer<EProjectKey>
{

    @Override
    public EProjectKey deserialize( final JsonElement src, final Type type, final JsonDeserializationContext ctx )
        throws JsonParseException
    {
        final JsonObject obj = src.getAsJsonObject();
        final JsonElement pv = obj.get( SerializationConstants.PROJECT_VERSION );
        final ProjectVersionRef ref = ctx.deserialize( pv, ProjectVersionRef.class );

        final JsonElement pf = obj.get( SerializationConstants.PROJECT_FACTS );
        if ( pf == null )
        {
            return new EProjectKey( ref );
        }

        final EGraphFacts facts = ctx.deserialize( pf, EGraphFacts.class );
        return new EProjectKey( ref, facts );
    }

    @Override
    public JsonElement serialize( final EProjectKey src, final Type type, final JsonSerializationContext ctx )
    {
        final JsonObject obj = new JsonObject();
        obj.add( SerializationConstants.PROJECT_VERSION, ctx.serialize( src.getProject() ) );

        final EGraphFacts facts = src.getFacts();
        if ( facts != null )
        {
            obj.add( SerializationConstants.PROJECT_FACTS, ctx.serialize( src.getFacts() ) );
        }

        return obj;
    }

}
