package org.commonjava.tensor.io.json;

import java.lang.reflect.Type;
import java.util.Collection;
import java.util.Set;

import org.apache.maven.graph.effective.EProjectCycle;
import org.apache.maven.graph.effective.EProjectWeb;
import org.apache.maven.graph.effective.rel.ProjectRelationship;
import org.apache.maven.graph.spi.GraphDriverException;
import org.apache.maven.graph.spi.effective.EGraphDriver;

import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParseException;
import com.google.gson.JsonSerializationContext;

public class EProjectWebSer
    extends AbstractProjectNetSer<EProjectWeb>
{

    private final EGraphDriver driver;

    public EProjectWebSer( final EGraphDriver driver )
    {
        this.driver = driver;
    }

    @Override
    public EProjectWeb deserialize( final JsonElement src, final Type typeInfo, final JsonDeserializationContext ctx )
        throws JsonParseException
    {
        final JsonObject obj = src.getAsJsonObject();

        final Collection<ProjectRelationship<?>> rels = deserializeRelationships( obj, ctx );
        final Set<EProjectCycle> cycles = deserializeCycles( obj, ctx );

        EProjectWeb web;
        try
        {
            web = new EProjectWeb( driver.newInstance() );
        }
        catch ( final GraphDriverException e )
        {
            throw new JsonParseException( "Cannot get graph database sub-driver: " + e.getMessage(), e );
        }

        web.addAll( rels );

        for ( final EProjectCycle cycle : cycles )
        {
            web.addCycle( cycle );
        }

        return web;
    }

    @Override
    public JsonElement serialize( final EProjectWeb src, final Type typeInfo, final JsonSerializationContext ctx )
    {
        final JsonObject obj = new JsonObject();

        serializeRelationships( src, obj, ctx );
        serializeCycles( src, obj, ctx );

        return obj;
    }

}
