package org.commonjava.tensor.io.json;

import java.lang.reflect.Type;

import org.apache.maven.graph.common.DependencyScope;
import org.apache.maven.graph.common.RelationshipType;
import org.apache.maven.graph.common.ref.ArtifactRef;
import org.apache.maven.graph.common.ref.ProjectRef;
import org.apache.maven.graph.common.ref.ProjectVersionRef;
import org.apache.maven.graph.effective.rel.DependencyRelationship;
import org.apache.maven.graph.effective.rel.ExtensionRelationship;
import org.apache.maven.graph.effective.rel.ParentRelationship;
import org.apache.maven.graph.effective.rel.PluginDependencyRelationship;
import org.apache.maven.graph.effective.rel.PluginRelationship;
import org.apache.maven.graph.effective.rel.ProjectRelationship;

import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParseException;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;

public class ProjectRelationshipSer
    implements JsonSerializer<ProjectRelationship<?>>, JsonDeserializer<ProjectRelationship<?>>
{

    @Override
    public ProjectRelationship<?> deserialize( final JsonElement src, final Type typeInfo,
                                               final JsonDeserializationContext ctx )
        throws JsonParseException
    {
        final JsonObject obj = src.getAsJsonObject();

        // TODO: Make allowances for migration from older serialized objects!
        //        final int jsonVersion = obj.get( SerializationConstants.JSON_VERSION )
        //                                   .getAsInt();
        //
        //        if ( jsonVersion != SerializationConstants.CURRENT_JSON_VERSION )
        //        {
        //        }

        final String type = obj.get( SerializationConstants.RELATIONSHIP_TYPE )
                               .getAsString();
        final String decl = obj.get( SerializationConstants.DECLARING_REF )
                               .getAsString();

        final RelationshipType rt = RelationshipType.getType( type );

        final ProjectVersionRef declaring = JsonUtils.parseProjectVersionRef( decl );

        switch ( rt )
        {
            case DEPENDENCY:
            {
                final String tgt = getRawTarget( obj );
                final ArtifactRef target = JsonUtils.parseArtifactRef( tgt );

                final JsonElement scp = obj.get( SerializationConstants.SCOPE );
                final DependencyScope scope =
                    scp == null ? DependencyScope.compile : DependencyScope.getScope( scp.getAsString() );

                return new DependencyRelationship( declaring, target, scope, getIndex( obj ), isManaged( obj ) );
            }
            case EXTENSION:
            {
                final String tgt = getRawTarget( obj );
                final ProjectVersionRef target = JsonUtils.parseProjectVersionRef( tgt );

                return new ExtensionRelationship( declaring, target, getIndex( obj ) );
            }
            case PARENT:
            {
                final String tgt = getRawTarget( obj );
                final ProjectVersionRef target = JsonUtils.parseProjectVersionRef( tgt );

                return new ParentRelationship( declaring, target );
            }
            case PLUGIN:
            {
                final String tgt = getRawTarget( obj );
                final ProjectVersionRef target = JsonUtils.parseProjectVersionRef( tgt );

                return new PluginRelationship( declaring, target, getIndex( obj ), isManaged( obj ) );
            }
            case PLUGIN_DEP:
            {
                final String plug = obj.get( SerializationConstants.PLUGIN_REF )
                                       .getAsString();
                final ProjectRef plugin = JsonUtils.parseProjectRef( plug );

                final String tgt = getRawTarget( obj );
                final ArtifactRef target = JsonUtils.parseArtifactRef( tgt );

                return new PluginDependencyRelationship( declaring, plugin, target, getIndex( obj ), isManaged( obj ) );
            }
            default:
            {
                throw new JsonParseException( "Unknown ProjectRelationship type: '" + rt + "'." );
            }
        }
    }

    private boolean isManaged( final JsonObject obj )
    {
        final JsonElement mgd = obj.get( SerializationConstants.MANAGED );
        return mgd == null ? false : mgd.getAsBoolean();
    }

    private int getIndex( final JsonObject obj )
    {
        final JsonElement idx = obj.get( SerializationConstants.INDEX );
        return idx == null ? -1 : idx.getAsInt();
    }

    private String getRawTarget( final JsonObject obj )
    {
        return obj.get( SerializationConstants.TARGET_REF )
                  .getAsString();
    }

    @Override
    public JsonElement serialize( final ProjectRelationship<?> src, final Type type, final JsonSerializationContext ctx )
    {
        final JsonObject obj = new JsonObject();
        obj.addProperty( SerializationConstants.JSON_VERSION, SerializationConstants.CURRENT_JSON_VERSION );

        obj.addProperty( SerializationConstants.RELATIONSHIP_TYPE, src.getType()
                                                                      .name() );
        obj.addProperty( SerializationConstants.DECLARING_REF, JsonUtils.formatRef( src.getDeclaring() ) );
        obj.addProperty( SerializationConstants.TARGET_REF, JsonUtils.formatRef( src.getTarget() ) );
        obj.addProperty( SerializationConstants.INDEX, src.getIndex() );

        switch ( src.getType() )
        {
            case DEPENDENCY:
            {
                obj.addProperty( SerializationConstants.SCOPE, ( (DependencyRelationship) src ).getScope()
                                                                                               .realName() );
                break;
            }
            case PLUGIN_DEP:
            {
                obj.addProperty( SerializationConstants.PLUGIN_REF,
                                 JsonUtils.formatRef( ( (PluginDependencyRelationship) src ).getPlugin() ) );
                break;
            }
            default:
            {
            }
        }

        return obj;
    }

}
